var logger = require('convoi-logger');
var EventEmitter = require('events').EventEmitter;
var MongoClient = require('mongodb').MongoClient;
var util = require('util');

var ConvoiMongoClient = function() {
    EventEmitter.call(this);
    this.connections = {};
};

util.inherits(ConvoiMongoClient, EventEmitter);

ConvoiMongoClient.prototype.connect = function(url, options, callback) {
    if(typeof options == 'function') {
        callback = options;
        options = null;
    }

    var connection = this.connections[url];
    if (connection && connection.connected) {
        logger.debug('found existing connection for url: '+  url);
        return process.nextTick(function() {
            callback(null, connection.db);
        });
    } else if (connection) {
        logger.debug('waiting on existing connection for url: '+  url);
        return this.on('connected', function(_connection) {
            if (_connection.url == url) {
                logger.debug('existing connection established for url: '+  url);
                callback(null, _connection.db);
            }
        });
    }

    logger.debug('establishing new connection for url: '+  url);

    var self = this;
    self.connections[url] = {url: url, connected: false};
    MongoClient.connect(url, options, function(err, db) {
        if (err) {
            // TODO: emit error?
            logger.error('error connecting to MongoDB', err);
            return callback(err);
        }

        logger.debug('new connection established for url: '+  url);

        var _connection = {url: url, connected: true, db: db};
        self.connections[url] = _connection;
        self.emit('connected', _connection);
        callback(null, db);
    });
};

module.exports = new ConvoiMongoClient();
